//////////////////////////////////////////////
// main.cpp
//
//////////////////////////////////////////////

/// Includes ---------------------------------

// nkLog
#include <NilkinsLog/ConsoleLogger.h>

// nkGraphics
#include <NilkinsGraphics/Log/LogManager.h>
#include <NilkinsGraphics/RenderContexts/RenderContextManager.h>
#include <NilkinsGraphics/Systems/MainSystem.h>

// Standards
#include <memory>

/// Function ---------------------------------

int main ()
{
	// Prepare logging capabilities
	std::unique_ptr<nkLog::Logger> logger = std::make_unique<nkLog::ConsoleLogger>() ;
	nkGraphics::LogManager::getInstance()->setReceiver(logger.get()) ;

	// Initialize the component
	if (!nkGraphics::MainSystem::getInstance()->initialize())
		return -1 ;

	// RenderContexts are environment in which it is possible to render
	// They can render to a window, or be offscreen
	// Main point is that it is possible to setup some parameters like their compositor, size, and more
	// This call create a render context and its window and link both of them together
	nkGraphics::RenderContext* context = nkGraphics::RenderContextManager::getInstance()->createRenderContextAndWindowFromConfig() ;

	// Then rendering can use that context to know where to render and what setup to use
	nkGraphics::MainSystem::getInstance()->run(context) ;

	// Then kill the instance, which will automatically clean its method through the shutdown method
	nkGraphics::MainSystem::getInstance()->kill() ;

	return 0 ;
}