﻿//////////////////////////////////////////////
// PbsMaterialInfo.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkAstraeus
{
	enum class ONE_CHANNEL_PACK ;
	enum class THREE_CHANNEL_PACK ;
}

namespace nkGraphics
{
	class Texture ;
}

/// Includes ---------------------------------

// nkMaths
#include <NilkinsMaths/Algebra/Vector.h>

/// Class ------------------------------------

namespace nkAstraeus
{
	struct PbsMaterialInfo
	{
		// Structures
		nkMaths::Vector _lightDir = nkMaths::Vector(1.f, -1.f, 1.f) ;
		nkMaths::Vector _lightColor ;
		nkMaths::Vector _albedoColor ;
		nkMaths::Vector _specularColor = nkMaths::Vector(0.5f, 0.5f, 0.5f) ;
		nkMaths::Vector _emissiveColor ;
		nkMaths::Vector _envColor ;

		unsigned int _albedoColorSlot = 0 ;
		unsigned int _specularColorSlot = 0 ;
		unsigned int _emissiveColorSlot = 0 ;
		unsigned int _envColorSlot = 0 ;

		float _metalness = 0.5f ;
		float _roughness = 0.5f ;
		
		unsigned int _metalnessSlot = 0 ;
		unsigned int _roughnessSlot = 0 ;

		// Pointers
		nkGraphics::Texture* _albedoTex = nullptr ;
		nkGraphics::Texture* _specularTex = nullptr ;
		nkGraphics::Texture* _emissiveTex = nullptr ;
		nkGraphics::Texture* _envTex = nullptr ;
		nkGraphics::Texture* _irradianceTex = nullptr ;
		nkGraphics::Texture* _metalnessTex = nullptr ;
		nkGraphics::Texture* _roughnessTex = nullptr ;
		nkGraphics::Texture* _normalMapTex = nullptr ;

		// Packing
		THREE_CHANNEL_PACK _albedoPacking ;
		THREE_CHANNEL_PACK _specularPacking ;
		THREE_CHANNEL_PACK _emissivePacking ;
		THREE_CHANNEL_PACK _envPacking ;
		THREE_CHANNEL_PACK _irradiancePacking ;
		ONE_CHANNEL_PACK _metalnessPacking ;
		ONE_CHANNEL_PACK _roughnessPacking ;
		THREE_CHANNEL_PACK _normalMapPacking ;

		// Flags
		bool _envTexIsCubeByDefault = true ;
		bool _irradianceTexIsCubeByDefault = true ;

		// Loading witnesses for updates
		bool _usingAlbedoTex = false ;
		bool _usingSpecularTex = false ;
		bool _usingEmissiveTex = false ;
		bool _usingEnvTex = false ;
		bool _usingIrradianceTex = false ;
		bool _usingMetalnessTex = false ;
		bool _usingRoughnessTex = false ;
		bool _usingNormalMapTex = false ;
	} ;
}