﻿//////////////////////////////////////////////
// CompositorManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Compositor ;
	class RenderContext ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <mutex>
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkGraphics
{	
	class DLL_GRAPHICS_EXPORT CompositorManager final : public nkCommon::SingletonClass<CompositorManager>
	{
		friend class nkCommon::SingletonClass<CompositorManager> ;

		private :
		
			// Attributs
			// Une map des Compositor enregistrés
			std::unordered_map<std::string, Compositor*> _memory ;

			// Threading
			std::mutex _mutex ;

			// Le compositor actif
			Compositor* _activeCompositor ;
			// Le Compositor par défaut
			Compositor* _defaultCompositor ;
		
		public :
		
			// Constructeur, destructeur
			CompositorManager () ;
			~CompositorManager () ;

			// Getters
			Compositor* getActiveCompositor () const ;
			Compositor* getDefaultCompositor () const ;
			
			// Enregistrement
			Compositor* createOrRetrieve (const std::string_view& name) ;
			Compositor* get (const std::string_view& name) ;
			Compositor* getByIndex (unsigned int index) ;
			void rename (const std::string_view& oldName, const std::string_view& newName) ;
			void erase (const std::string_view& name) ;

			// Setters
			void setActiveCompositor (Compositor* compositor) ;
			void setDefaultCompositor (Compositor* compositor) ;

			// Activations lors des changements de contextes
			void prepareForContext (RenderContext* context) ;
	} ;
}