﻿//////////////////////////////////////////////
// NodeManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Node ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <mutex>
#include <unordered_map>
#include <string>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT NodeManager final : public nkCommon::SingletonClass<NodeManager>
	{
		friend class nkCommon::SingletonClass<NodeManager> ;

		private :

			// Attributes
			// Nodes currently in memory
			std::unordered_map<std::string, Node*> _nodeMap ;

			// Threading
			mutable std::mutex _mutex ;

			// Counter
			unsigned int _nodeCounter ;

		private :

			// Functions
			// Constructor
			NodeManager () ;
		
		public :

			// Destructor
			~NodeManager () ;
	
			// Memory manipulation
			Node* create () ;
			Node* createOrRetrieve (const std::string_view& name) ;
			Node* get (const std::string_view& name) const ;
			Node* getByIndex (unsigned int index) const ;
			void rename (const std::string_view& currentName, const std::string_view& newName) ;
			void erase (const std::string_view& name) ;
	} ;
}