﻿//////////////////////////////////////////////
// RenderContextManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class RenderContext ;

	struct RenderContextDescriptor ;
}

namespace nkWinUi
{
	class GraphicsWindow ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <mutex>
#include <unordered_map>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT RenderContextManager final : public nkCommon::SingletonClass<RenderContextManager>
	{
		friend class nkCommon::SingletonClass<RenderContextManager> ;

		private :
		
			// Attributs
			std::unordered_map<nkWinUi::GraphicsWindow*, RenderContext*> _contextMap ;

			// Contexte activé
			RenderContext* _activatedContext ;
			// Contexte par défaut
			RenderContext* _dummyContext ;

			// Threading
			mutable std::mutex _mutex ;

		private :

			// Functions
			// Constructor
			RenderContextManager () ;
		
		public :
		
			// Destructor
			~RenderContextManager () ;

			// Getter
			const std::unordered_map<nkWinUi::GraphicsWindow*, RenderContext*>& getRenderContextMap () const ;
			RenderContext* getRenderContextOf (nkWinUi::GraphicsWindow* win) const ;
			RenderContext* getActiveRenderContext () const ;
			RenderContext* getDefaultRenderContext () const ;
			nkWinUi::GraphicsWindow* getWindowOf (RenderContext* context) const ;

			// Enregistrement
			RenderContext* createRenderContextFor (nkWinUi::GraphicsWindow* win) ;
			RenderContext* createRenderContextAndWindow (const RenderContextDescriptor& contextDesc) ;
			RenderContext* createRenderContextAndWindowFromConfig () ;

			// Activation
			void activateContextOf (nkWinUi::GraphicsWindow* win) ;
			void activateContext (RenderContext* context) ;

			// Fin de vie
			void eraseFor (nkWinUi::GraphicsWindow* win) ;
	} ;
}