﻿//////////////////////////////////////////////
// BufferCast.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkMemory
#include "Buffer.h"

/// Class ------------------------------------

namespace nkMemory
{
	template <typename T = unsigned char>
	class BufferCast
	{
		private :

			// Attributes
			// Base buffer
			Buffer _buffer ;

		public :

			// Constructors
			BufferCast () = default ;
			
			BufferCast (unsigned long long size)
			:	_buffer(size * sizeof(T))
			{
				// Nothing to do
			}

			BufferCast (T* data, unsigned long long size)
			:	_buffer ((unsigned char*)data, size * sizeof(T))
			{
				// Nothing to do
			}

			BufferCast (const Buffer& buffer)
			:	_buffer (buffer)
			{
				// Nothing to do
			}

			BufferCast (Buffer&& buffer)
			:	_buffer (std::move(buffer))
			{
				// Nothing to do
			}

			BufferCast (const BufferCast& cast)
			:	_buffer (cast._buffer)
			{
				// Nothing to do
			}

			BufferCast (BufferCast&& cast)
			:	_buffer (std::move(cast._buffer))
			{
				// Nothing to do
			}

			// Destructor
			~BufferCast () = default ;

		public :

			// Getters
			T* getData () const
			{
				return (T*)_buffer.getData() ;
			}

			unsigned long long getSize () const
			{
				return _buffer.getSize() / sizeof(T) ;
			}

			T& front ()
			{
				return getData()[0] ;
			}

			T& back ()
			{
				return getData()[getSize() - 1] ;
			}

		public :

			// Management
			void clear ()
			{
				_buffer.clear() ;
			}

			void resize (unsigned long long size)
			{
				_buffer.resize(size * sizeof(T)) ;
			}

			T& append (const T& value)
			{
				unsigned long long size = getSize() ;
				resize(size + 1) ;

				T* data = getData() ;
				data[size] = value ;

				return data[size] ;
			}

		public :

			// Operators
			T& operator[] (unsigned long long index)
			{
				return getData()[index] ;
			}

			const T& operator[] (unsigned long long index) const
			{
				return getData()[index] ;
			}			

		public :

			// Templated constructors
			BufferCast (const std::vector<T>& vec)
			:	_buffer (vec)
			{
				// Nothing to do
			}
	} ;
}