﻿//////////////////////////////////////////////
// TabWindow.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkWinUi
{
	class Window ;
}

/// Includes ---------------------------------

// nkWinUi
#include "Component.h"

#include "../Dll/DllDefines.h"

#include "../Inputs/CallbackRepository.h"

// Standards
#include <map>
#include <string>
#include <unordered_map>

/// Class ------------------------------------
	
namespace nkWinUi
{
	class DLL_WINUI_EXPORT TabWindow : public Component
	{
		public :

			// La classe des tabs pour garder les infos dont elles ont besoin, de base
			class DLL_WINUI_EXPORT Tab
			{
				protected :
						
					// Le titre
					std::string _label ;
					// La Window associee
					Window* _associatedWin ;
					// Son id
					int _id ;

					// La tab window parente
					TabWindow* _parentWindow ;

				public :

					// Constructeur, destructeur
					Tab () ;
					virtual ~Tab () ;

					// Getters
					const char* getLabel () const ;
					Window* getAssociatedWin () const ;
					int getId () const ;
					TabWindow* getParentWindow () const ;

					// Setters
					virtual void setLabel (const std::string_view& label, bool autoUpdate = true) ;
					void setAssociatedWin (Window* win) ;
					void setId (int id) ;
					void setParentWindow (TabWindow* parent) ;					
			} ;

		protected :

			// Functions
			virtual Tab* _allocateTab () const = 0 ;

		protected :
		
			// Attributs
			// Les tab et leurs Window associees
			std::map<std::string, Tab*, std::less<>> _tabMemory ;
			std::map<int, Tab*> _indicesMemory ;

			// La tab active
			Tab* _activeTab ;

			// Callback
			tabWindowCallback _callback ;
		
		public :
		
			// Constructeur, destructeur
			TabWindow () ;
			~TabWindow () ;

			// Pour les tabs
			virtual Tab* addTab (const std::string_view& label) ;
			Tab* getTabByLabel (const std::string_view& label) const ;
			Tab* getTabById (int id) const ;
			virtual void changeTabLabel (const std::string_view& oldLabel, const std::string_view& newLabel) ;
			virtual void eraseTab (const std::string_view& label) ;

			// Pour les layouts en eux même
			virtual Layout* setLayout (LAYOUT_TYPE layoutType) override ;

			// Activité
			virtual void setActiveTab (Tab* toActivate) ;
			Tab* getActiveTab () const ;

			// Tailles
			virtual void adaptToDisplayArea (Window* toAdapt) const = 0 ;

			// Callback
			void setTabSpecializedCallback (tabWindowCallback value) ;
			tabWindowCallback getTabSpecializedCallback () const ;

			// Import / Export
			virtual void exportIntrospection (nkExport::Node* rootNode) override ;
			void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}